@@ -21,3 +21,7 @@ CREATE FUNCTION mroonga_escape RETURNS STRING
 DROP FUNCTION IF EXISTS mroonga_snippet_html;
 CREATE FUNCTION mroonga_snippet_html RETURNS STRING
   SONAME 'ha_mroonga.so';
+
+DROP FUNCTION IF EXISTS mroonga_normalize;
+CREATE FUNCTION mroonga_normalize RETURNS STRING
+  SONAME 'ha_mroonga.so';
